const fs = require('fs')
const path = require('path')
const { tmpdir } = require('os')
const Crypto = require('crypto')
const webp = require('node-webpmux')
const FileType = require('file-type')
const { imageToWebp, videoToWebp } = require('./library/exit') // pastikan kamu punya ini

async function writeExif(media, data) {
    const anu = await FileType.fromBuffer(media)
    const wMedia = /webp/.test(anu.mime) ? media :
        /jpeg|jpg|png/.test(anu.mime) ? await imageToWebp(media) :
        /mp4|video/.test(anu.mime) ? await videoToWebp(media) : ''

    const tmpFileIn = path.join(tmpdir(), `${Crypto.randomBytes(6).readUIntLE(0, 6).toString(36)}.webp`)
    const tmpFileOut = path.join(tmpdir(), `${Crypto.randomBytes(6).readUIntLE(0, 6).toString(36)}.webp`)

    fs.writeFileSync(tmpFileIn, wMedia)

    if (!data) return tmpFileIn

    const img = new webp.Image()
    const wr = {
        a: global.author || '',
        b: data.packname || global.packname || '',
        c: data.author || global.author || '',
        d: data.categories || [''],
        e: data.isAvatar || 0
    }

    const json = {
        'sticker-pack-id': wr.a,
        'sticker-pack-name': wr.b,
        'sticker-pack-publisher': wr.c,
        'emojis': wr.d,
        'is-avatar-sticker': wr.e
    }

    const exifAttr = Buffer.from([0x49, 0x49, 0x2A, 0x00, 0x08, 0x00, 0x00, 0x00,
        0x01, 0x00, 0x41, 0x57, 0x07, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x16, 0x00, 0x00, 0x00])
    const jsonBuff = Buffer.from(JSON.stringify(json), 'utf-8')
    const exif = Buffer.concat([exifAttr, jsonBuff])
    exif.writeUIntLE(jsonBuff.length, 14, 4)

    await img.load(tmpFileIn)
    fs.unlinkSync(tmpFileIn)
    img.exif = exif
    await img.save(tmpFileOut)

    return tmpFileOut
}

module.exports = { writeExif }
